;
; Stair Interpolation, V2.4
;
; Martin Egger (martin.egger@gmx.net)
; (C) 2007, Bern, Switzerland
;
; The image will be enlarged in 10% steps up to the specified factor using 
; best available (=Lanczos) interpolation.
;
; This plugin was tested with Gimp 2.4
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;
; Define the function
;
(define (script-fu-Eg-StairInterpolation InImage InLayer InFactor)
;		
	(let*	(
		(TheImage (car (gimp-image-duplicate InImage)))
		(TheLayer (car (gimp-image-flatten TheImage)))
		(Step (* 10 (- InFactor 1)))
		(TheWidth (car (gimp-image-width TheImage)))
		(TheHeight (car (gimp-image-height TheImage)))
		(TotWidth (* InFactor TheWidth))
		(TotHeight (* InFactor TheHeight))
		(IncWidth (/ (- TotWidth TheWidth) Step))
		(IncHeight (/ (- TotHeight TheHeight) Step))
		(Counter 1)
		)
		(gimp-image-undo-disable TheImage)
		(gimp-selection-none TheImage)
;;;
; Temporary commented out to avoid a major bug in the transform code (bugzilla #317853)
;
; Resize the image
;
;		(while (<= Counter Step)
;
;			(let*	(
;				(NewWidth (+ TheWidth (* Counter IncWidth)))
;				(NewHeight (+ TheHeight (* Counter IncHeight)))
;				)
;				(gimp-drawable-transform-scale TheLayer 0.0 0.0 NewWidth NewHeight TRANSFORM-FORWARD INTERPOLATION-LANCZOS TRUE 3 FALSE)
;
;
;			(set! Counter (+ Counter 1))
;		)
;
; End of comment out section
;;;
;;;
; Temporary fix to avoid a major bug in the transform code (bugzilla #317853)
; do a direct transformation to the final size, not a stair interpolation
;
		(gimp-drawable-transform-scale TheLayer 0.0 0.0 TotWidth TotHeight TRANSFORM-FORWARD INTERPOLATION-LANCZOS TRUE 3 FALSE)
;
; End of temporary fix
;;;
		(gimp-image-resize-to-layers TheImage)
		(gimp-display-new TheImage)
		(gimp-drawable-set-name TheLayer "StairInterpolated")
		(gimp-image-undo-enable TheImage)
	)
;
; Finish work
;
	(gimp-displays-flush)
;
)
;
; Register the function with the GIMP
;
(script-fu-register
	"script-fu-Eg-StairInterpolation"
	_"S_tair Interpolation"
	"Resize images using Stair Inperpolation with Lanczos algorithm"
	"Martin Egger (martin.egger@gmx.net)"
	"Martin Egger, Bern, Switzerland"
	"07.10.2007"
	"RGB* GRAY*"
	SF-IMAGE	"The Image"	0
	SF-DRAWABLE	"The Layer"	0
	SF-ADJUSTMENT	"Factor"	'(1.1 1.00001 5.0 0.1 0 2 0)
)
;
(script-fu-menu-register "script-fu-Eg-StairInterpolation"
			 "<Image>/Photo/B/Photo Tools")
;
